import * as vscode from 'vscode';
import * as json from 'jsonc-parser';
import * as path from 'path';
import { notDeepStrictEqual } from 'assert';
import {defaultFieldTypes} from './utils'
import {createTranslationObject} from "@aemforms/af-core";

type FieldType = {
    name: string,
    type: string,
    hasItems: boolean
    offset: number,
    length: number
}

type X = {[key: string]: json.Node}

const fieldTypes = [
    "text-input",
    "number-input",
    "date-input",
    "file-input",
    "multiline-input",
    "drop-down",
    "radio-button",
    "checkbox-group",
    "checkbox",
    "plain-text",
    "submit",
    "panel"
]

const defaults = {
    "all" : {
        "name" : "new_field",
        "label" : {
            "value" : ""
        },
        "description": "",
        "type" : "string",
        "required" : false
    },
    'number-input' : {
        "type" : "number",
        "minimum" : 0,
        "maximum" : 100
    },
    'date-input' : {
        "type" : "string",
        "minimum" : "2020-10-10",
        "maximum" : "2021-10-10",
        "format" : "date"
    },
    'file-input' : {
        "type" : "file",
        "maxFileSize" : "2MB",
        "accept" : ["image/*"]
    },
    "checkbox" : {
        "type" : "boolean",
        "enum" : [true, false],
    },
    "checkbox-group" : {
        "enum" : [],
        "enumNames" : []
    },
    "radio-button" : {
        "enum" : [],
        "enumNames" : []
    },
    "drop-down" : {
        "enum" : [],
        "enumNames" : []
    },
    "panel" : {
        "type" : "object",
        "items" : []
    },
    "submit"  : {
        "type" : "button",
        "label"  : {
            "value" : "Submit"
        },
        "events" : {
            "click" : "dispatch_event($form, 'submit')"
        }
    }
}

export class FormTreeProvider implements vscode.TreeDataProvider<FieldType> {

    private _onDidChangeTreeData: vscode.EventEmitter<FieldType | null> = new vscode.EventEmitter<FieldType | null>();
    readonly onDidChangeTreeData: vscode.Event<FieldType | null> = this._onDidChangeTreeData.event;

    private tree: json.Node;
    private text: string;
    private editor: vscode.TextEditor;
    private autoRefresh = true;

    constructor(private context: vscode.ExtensionContext) {
        vscode.window.onDidChangeActiveTextEditor(() => this.onActiveEditorChanged());
        vscode.workspace.onDidChangeTextDocument(e => this.onDocumentChanged(e));
        this.autoRefresh = vscode.workspace.getConfiguration('formTree').get('autorefresh');
        vscode.workspace.onDidChangeConfiguration(() => {
            this.autoRefresh = vscode.workspace.getConfiguration('formTree').get('autorefresh');
        });
        this.onActiveEditorChanged();
    }

    refresh(offset?: FieldType): void {
        this.parseTree();
        if (offset) {
            this._onDidChangeTreeData.fire(offset);
        } else {
            this._onDidChangeTreeData.fire(undefined);
        }
    }

    delete(offset?: FieldType): void {
        this.editor.edit(editBuilder => {
            const path = json.getLocation(this.text, offset.offset).path;
            let fieldNode = json.findNodeAtLocation(this.tree, path);
            const parent = fieldNode.parent
            const edits = json.modify(this.text, path, undefined, {})
            edits.forEach(e => {
                const range = new vscode.Range(this.editor.document.positionAt(e.offset), this.editor.document.positionAt(e.offset + e.length));
                editBuilder.replace(range, "");
            })
            setTimeout(() => {
                this.parseTree();
                this.refresh(undefined);
            }, 100);
        });
    }

    translate(): void {
        this.editor.edit(editBuilder => {
            this.text = this.editor.document.getText();
            var formObj = JSON.parse(this.text);
            const [newObj, langObj] = createTranslationObject(formObj);
            // update whole text
            var lineCount = this.editor.document.lineCount;
            var lastLine = this.editor.document.lineAt(lineCount - 1);
            var endPos = lastLine.range.end;
            var range = new vscode.Range(new vscode.Position(0, 0), endPos);
            editBuilder.delete(range);
            editBuilder.insert(new vscode.Position(0,0), JSON.stringify(newObj, null, 2));
            // write the dictionary to the file system using the name of the current file
            const wsedit = new vscode.WorkspaceEdit();
            const wsPath = vscode.workspace.workspaceFolders?.[0].uri.fsPath; // gets the path of the first workspace folder
            const currentFile = path.basename(this.editor.document.fileName);
            const extension = path.extname(currentFile)
            const newExtension = `.i18n${extension}`;
            const newFile = currentFile.replace(extension, newExtension);
            const dictionaryPath = path.join(wsPath, "generated", "__localization__", newFile);
            const dictionaryFileUri = vscode.Uri.file(dictionaryPath);
            const content = JSON.stringify(langObj, null, 2)
            var uint8Content: Uint8Array = new TextEncoder().encode(content);
            //@ts-ignore
            vscode.workspace.fs.writeFile(dictionaryFileUri, uint8Content, {create: true, overwrite: true}).then(() => {
                vscode.workspace.openTextDocument(dictionaryFileUri).then((t) => {
                    t.save().then(() => {
                        vscode.workspace.applyEdit(wsedit).then(() => {
                            this.editor.document.save().then(() => {
                                setTimeout(() => {
                                    alert(1)
                                    this.parseTree();
                                    this.refresh(undefined);
                                }, 100);
                               
                            });
                        });    
                    })
                })
            })
        });
    }

    getFieldType() {
        return vscode.window.showQuickPick(fieldTypes, { title: 'Select the field type' }).then(x => {
            return {
                fieldType: x || 'text-input'
            }
        })
    }

    getFieldName(obj) {
        return vscode.window.showInputBox({ title: 'Select the field name', value : "name", prompt : "Only alphanumeric characters allowed" }).then(x => {
            return {
                ...obj,
                name: x || "name"
            }
        })
    }

    getFieldLabel(obj) {
        return vscode.window.showInputBox({ title: 'Label of the field', value : "Label" }).then(x => {
            return {
                ...obj,
                label: {
                    value: x || "Field Label"
                }
            }
        })
    }

    add(offset?: FieldType): void {
            this.getFieldType().then(this.getFieldName).then(this.getFieldLabel)
            .then(value => {
                if (value !== null && value !== undefined) {
                    const newField = {
                        ...defaults.all,
                        ...(defaults[value.fieldType] ? defaults[value.fieldType] : {}),
                        ...value
                    }
                    const newFieldString = JSON.stringify(newField, null, 2)
                    this.editor.edit(editBuilder => {
                        const path = json.getLocation(this.text, offset.offset).path;
                        const index = path.pop()
                        path.push(<number>index + 1)
                        const edits = json.modify(this.text, path, newField, {isArrayInsertion: true})
                        edits.forEach(e => {
                            const range = new vscode.Range(this.editor.document.positionAt(e.offset), this.editor.document.positionAt(e.offset + e.length));
                            editBuilder.replace(range, e.content);
                        })
                        vscode.commands.executeCommand('editor.action.formatDocument')
                        this.select(new vscode.Range(this.editor.document.positionAt(edits[0].offset), this.editor.document.positionAt(edits[0].offset + edits[0].length)))
                        setTimeout(() => {
                            this.parseTree();
                            this.refresh(offset);
                        }, 100);                
                    });
                }
            });
    }

    rename(offset?: FieldType): void {
        vscode.window.showInputBox({ placeHolder: 'Enter the new label' })
            .then(value => {
                if (value !== null && value !== undefined) {
                    this.editor.edit(editBuilder => {
                        const path = json.getLocation(this.text, offset.offset).path;
                        let fieldNode = json.findNodeAtLocation(this.tree, path);
                        let nameNode = this.getProperty(fieldNode, 'name')
                        if (nameNode) {
                            const range = new vscode.Range(this.editor.document.positionAt(nameNode.offset), this.editor.document.positionAt(nameNode.offset + nameNode.length));
                            editBuilder.replace(range, `"${value}"`);
                            setTimeout(() => {
                                this.parseTree();
                                this.refresh(offset);
                            }, 100);                       
                        } else {
                            vscode.window.showErrorMessage("You need to first add a name property");
                        }
                    });
                }
            });
    }

    private onActiveEditorChanged(): void {
        if (vscode.window.activeTextEditor) {
            if (vscode.window.activeTextEditor.document.uri.scheme === 'file') {
                const enabled = vscode.window.activeTextEditor.document.languageId === 'json' || vscode.window.activeTextEditor.document.languageId === 'jsonc';
                vscode.commands.executeCommand('setContext', 'formTreeEnabled', enabled);
                if (enabled) {
                    this.refresh();
                }
            }
        } else {
            vscode.commands.executeCommand('setContext', 'formTreeEnabled', false);
        }
    }

    private findFieldAtOffset(node: json.Node): FieldType {
        return undefined;
        //todo: make this work
        if (node.offset == 0) {
            return this.getForm(this.tree)[0]
        }
        const obj = this.getProperty(node, ['type', 'name', 'fieldType', 'items'])
        if (obj !== undefined) {
            return {
                offset: node.offset,
                length: node.length,
                name: '$form',
                type: 'Form',
                hasItems: true
            }
        }
        if (node.parent) {
            return this.findFieldAtOffset(node)
        }
    }

    private onDocumentChanged(changeEvent: vscode.TextDocumentChangeEvent): void {
        if (this.autoRefresh && changeEvent.document.uri.toString() === this.editor.document.uri.toString()) {
            for (const change of changeEvent.contentChanges) {
                const path = json.getLocation(this.text, this.editor.document.offsetAt(change.range.start)).path;
                path.pop();
                const node = path.length ? json.findNodeAtLocation(this.tree, path) : void 0;
                this.parseTree();
                this._onDidChangeTreeData.fire(this.findFieldAtOffset(node));
            }
        }
    }

    private parseTree(): void {
        this.text = '';
        this.tree = null;
        this.editor = vscode.window.activeTextEditor;
        if (this.editor && this.editor.document) {
            this.text = this.editor.document.getText();
            const tree = json.parseTree(this.text);
            if (this.isForm(tree)) {
                this.tree = tree
            }
        }
    }

    getChildren(offset?: FieldType): Thenable<FieldType[]> {
        let items: FieldType[] = []
        if (offset) {
            const path = json.getLocation(this.text, offset.offset).path;
            const node = json.findNodeAtLocation(this.tree, path);
            items = this.items(node)
        } else {
            items = this.tree ? this.getForm(this.tree) : []
        }
        return Promise.resolve(items)
    }

    private isElement(node: json.Node): boolean {
        return true
    }


    private getProperty(node: json.Node, properties: string|string[]) : any {
        let props = properties
        if (typeof props === 'string') {
            props = [props]
        }
        if (node.type === 'object') {
            const res = node.children.reduce((acc: X, x) => {
                const propertyName = x.children[0].value
                const isRequired = x.type === 'property' && props.indexOf(propertyName) > -1
                if (isRequired) {
                    acc[propertyName] = x.children[1]
                }
                return acc;
            }, {})
            if (Object.keys(res).length > 0) {
                if (typeof properties === 'string') {
                    return res[properties]
                }
                return res;
            }
        }
        return undefined
    }

    private isForm(node: json.Node) : boolean {
        const adaptiveform = this.getProperty(node, 'adaptiveform')
        return node.type === 'object' && adaptiveform != null
    }

    private items(node: json.Node) : FieldType[] {
        const itemsNode = this.getProperty(node, 'items')
        return itemsNode.children.map(i => {
            const obj = this.getProperty(i, ['type', 'name', 'fieldType', 'items', "enum", "format"])
            let fieldType;
            if (obj.fieldType === undefined) {
                const schema = {
                    type: obj.type?.value,
                    enum: obj.enum ? json.getNodeValue(obj.enum) : undefined,
                    format: obj.format?.value
                } 
                fieldType = defaultFieldTypes(schema)
            } else {
                fieldType = obj.fieldType.value
            }
            if (obj != undefined) {
                return {
                    name: obj.name?.value,
                    type: fieldType,
                    offset: i.offset,
                    length: i.length,
                    hasItems: obj.items !== undefined
                }
            }
        })
    }

    private getForm(node: json.Node): FieldType[] {
        const offset = {
            type: 'Form',
            name: '$form',
            offset: node.offset,
            length: node.length,
            hasItems: true
        }
        return [offset]
    }

    getTreeItem(field: FieldType): vscode.TreeItem {
        //const path = json.getLocation(this.text, offset).path;
        //const valueNode = json.findNodeAtLocation(this.tree, path);
        //if (valueNode) {
            const hasChildren = field.hasItems
            const treeItem: vscode.TreeItem = new vscode.TreeItem(this.getLabel(field), hasChildren ?  vscode.TreeItemCollapsibleState.Expanded : vscode.TreeItemCollapsibleState.None);
            treeItem.command = {
                command: 'extension.openJsonSelection',
                title: '',
                arguments: [new vscode.Range(this.editor.document.positionAt(field.offset), this.editor.document.positionAt(field.offset + field.length))]
            };
            treeItem.iconPath = this.getIcon(field);
            treeItem.contextValue = field.type;
            return treeItem;
        //}
        //return null;
    }

    select(range: vscode.Range) {
        this.editor.selection =  new vscode.Selection(range.start, range.end);
        this.editor.revealRange(range);
    }

    private getIcon(node: FieldType): any {
        const nodeType = node.type;
        return {
            light: this.context.asAbsolutePath(path.join('resources', 'light', 'boolean.svg')),
            dark: this.context.asAbsolutePath(path.join('resources', 'dark', 'boolean.svg'))
        };
        if (nodeType === 'boolean') {
            return {
                light: this.context.asAbsolutePath(path.join('resources', 'light', 'boolean.svg')),
                dark: this.context.asAbsolutePath(path.join('resources', 'dark', 'boolean.svg'))
            };
        }
        if (nodeType === 'string') {
            return {
                light: this.context.asAbsolutePath(path.join('resources', 'light', 'string.svg')),
                dark: this.context.asAbsolutePath(path.join('resources', 'dark', 'string.svg'))
            };
        }
        if (nodeType === 'number') {
            return {
                light: this.context.asAbsolutePath(path.join('resources', 'light', 'number.svg')),
                dark: this.context.asAbsolutePath(path.join('resources', 'dark', 'number.svg'))
            };
        }
        return null;
    }

    private getLabel(node: FieldType): string {
        return `${node.name} (${node.type})`
    }
}